/*
 * © NHN Corp. All rights reserved.
 * NHN Corp. PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 *
 * @author dl_platformsdk_all@nhn.com
 */
package com.toast.android.gamebase.sample;

import android.content.Context;
import android.content.Intent;
import android.os.AsyncTask;
import android.os.Build;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;

import com.toast.android.gamebase.Gamebase;

import java.lang.reflect.Method;
import java.util.concurrent.Executor;

public class GamebaseActivity extends AppCompatActivity {
    private static final String TAG = GamebaseActivity.class.getSimpleName();

    ////////////////////////////////////////////////////////////////////////////////
    //
    // Gamebase SDK Required
    //
    ////////////////////////////////////////////////////////////////////////////////
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        final String intentLog;
        if (data == null) {
            intentLog = "intent(null)";
        } else if (data.getExtras() == null) {
            intentLog = "extras(null)";
        } else {
            intentLog = data.getExtras().toString();
        }
        super.onActivityResult(requestCode, resultCode, data);
        android.util.Log.d(TAG, "onActivityResult:" + requestCode + ", " + resultCode + ", " + intentLog);

        Gamebase.onActivityResult(requestCode, resultCode, data);
    }

    ////////////////////////////////////////////////////////////////////////////////
    //
    // Push Analytics
    // http://docs.toast.com/en/Notification/Push/en/sdk-guide-android/#collect-indicators-gcm-only
    //
    ////////////////////////////////////////////////////////////////////////////////
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        initializeAsyncTask();
        onOpendPushAnalytics(getIntent());
    }

    @Override
    protected void onNewIntent(Intent intent) {
        final String intentLog;
        if (intent == null) {
            intentLog = "intent(null)";
        } else if (intent.getExtras() == null) {
            intentLog = "extras(null)";
        } else {
            intentLog = intent.getExtras().toString();
        }
        super.onNewIntent(intent);
        android.util.Log.d(TAG, "onNewIntent: " + intentLog);

        onOpendPushAnalytics(intent);
    }

    private void onOpendPushAnalytics(final Intent intent) {
        android.util.Log.d(TAG, "onOpendPushAnalytics()");

        try {
            Class<?> classPushAnalytics = Class.forName("com.toast.android.pushsdk.PushAnalytics");
            Method onOpened = classPushAnalytics.getDeclaredMethod("onOpened", Context.class, Intent.class);
            onOpened.invoke(null, this, intent);
            android.util.Log.v(TAG, "PushAnalytics.onOpened()");
        } catch (Exception e) {
            e.printStackTrace();
            android.util.Log.v(TAG, "PushSdk is not exist : " + e.getMessage());
        }
    }

    private void initializeAsyncTask() {
        // APIs 12 and below can cause problems with multithreading in AsyncTask
        // To solve this problem, single queue is forced from API 13.
        // Set a single queue even under API 12 to prevent problems.
        // API 10 and below are not meaningful because they do not use Executor.
        if (Build.VERSION.SDK_INT > 10 && Build.VERSION.SDK_INT < 13) {
            try {
                Method method = AsyncTask.class.getMethod("setDefaultExecutor", Executor.class);
                method.invoke(null, AsyncTask.SERIAL_EXECUTOR);
            } catch (Exception e) {
                android.util.Log.w(TAG, e.getMessage());
            }
        }

        // API Level 15 or less (4.0.3 IceCreamSandwichMR1) The first thread that calls AsyncTask in OS
        // An error will occur if it is not the main thread (UI Thread).
        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.JELLY_BEAN) {
            try {
                Class.forName("android.os.AsyncTask");
            } catch (ClassNotFoundException e) {
                e.printStackTrace();
                android.util.Log.w(TAG, e.getMessage());
            }
        }
    }
}