/*
 * © NHN Corp. All rights reserved.
 * NHN Corp. PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 *
 * @author dl_platformsdk_all@nhn.com
 */

package com.toast.android.gamebase.sample.ui;


import android.content.DialogInterface;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v4.content.ContextCompat;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.ListView;
import android.widget.Toast;

import com.toast.android.gamebase.Gamebase;
import com.toast.android.gamebase.GamebaseDataCallback;
import com.toast.android.gamebase.base.GamebaseException;
import com.toast.android.gamebase.base.log.Logger;
import com.toast.android.gamebase.base.purchase.PurchasableItem;
import com.toast.android.gamebase.base.purchase.PurchasableReceipt;
import com.toast.android.gamebase.base.purchase.PurchasableRetryTransactionResult;
import com.toast.android.gamebase.base.ui.SimpleSelectItemDialog;
import com.toast.android.gamebase.sample.GamebaseManager;
import com.toast.android.gamebase.sample.R;
import com.toast.android.gamebase.sample.customlist.CustomListViewAdapter;
import com.toast.android.gamebase.sample.util.AppUtil;

import java.util.ArrayList;
import java.util.List;

public class PurchaseFragment extends Fragment {

    private static final String TAG = PurchaseFragment.class.getSimpleName();
    private static View mCurrentView = null;
    private static List<PurchasableItem> mPurchasableItemList = new ArrayList<>();

    public PurchaseFragment() {
        // Required empty public constructor
    }


    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {

        setHasOptionsMenu(true);

        mCurrentView = inflater.inflate(R.layout.fragment_purchase, container, false);

        itemList();
        itemListOfNotConsumed();
        requestRetryTransaction();
        setPurchasableList();

        return mCurrentView;
    }

    ////////////////////////////////////////////////////////////////////////////////////////////////
    // Action Bar Item Setting
    @Override
    public void  onCreateOptionsMenu(Menu menu, MenuInflater inflater) {
        menu.clear();
        super.onCreateOptionsMenu(menu, inflater);

        inflater.inflate(R.menu.main, menu);

        MenuItem logoutItem = menu.findItem(R.id.action_logout);
        logoutItem.setVisible(false);
        MenuItem closeItem = menu.findItem(R.id.action_activityClose);
        closeItem.setVisible(true);
        MenuItem logOpenItem = menu.findItem(R.id.action_log);
        logOpenItem.setVisible(true);

    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        ((MainActivity)getActivity()).actionItemEvent(item);
        return super.onOptionsItemSelected(item);
    }
    ////////////////////////////////////////////////////////////////////////////////////////////////


    private void setPurchasableList() {

        AppUtil.showProgressDialog(getActivity(), "get Purchasable Item List...");
        //AppUtil.dismissProgressDialog(PurchaseActivity.this);

        final ListView listViewPurchasable = mCurrentView.findViewById(R.id.listview_purchase);
        final CustomListViewAdapter adapter = new CustomListViewAdapter();

        GamebaseManager.requestItemList(getActivity(), new GamebaseDataCallback<List<PurchasableItem>>() {
            @Override
            public void onCallback(List<PurchasableItem> result, GamebaseException exception) {

                if (Gamebase.isSuccess(exception)) {

                    mPurchasableItemList = result;
                    for (int i = 0; i < mPurchasableItemList.size(); i++) {
                        PurchasableItem item = mPurchasableItemList.get(i);
                        String itemName = item.itemName;
                        float itemPrice = item.price;
                        String itemCurrency = item.currency;
                        String marketItemId = item.marketItemId;
                        long itemSeq = item.itemSeq;
                        adapter.addItem(ContextCompat.getDrawable(getActivity(), R.drawable.item_001), itemName, itemCurrency + " " + String.valueOf(itemPrice));
                    }
                    listViewPurchasable.setAdapter(adapter);
                    AppUtil.dismissProgressDialog(getActivity());
                } else {
                    Gamebase.Util.showAlert(getActivity(), "Error", exception.toJsonString());
                    AppUtil.dismissProgressDialog(getActivity());

                    Logger.d(TAG, exception.toJsonString());
                    Logger.d(TAG, exception.getDomain());
                    Logger.d(TAG, String.valueOf(exception.getCode()));
                    Logger.d(TAG, exception.getMessage());
                    Logger.d(TAG, exception.getDetailDomain());
                    Logger.d(TAG, String.valueOf(exception.getDetailCode()));
                    Logger.d(TAG, exception.getDetailMessage());
                }
            }
        });


        listViewPurchasable.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView parent, View v, int position, long id) {

                PurchasableItem purchasableItem = mPurchasableItemList.get(position);
                Gamebase.Util.showToast(getActivity(), purchasableItem.toJsonString(), Toast.LENGTH_SHORT);

                AppUtil.showProgressDialog(getActivity(), "Purchase : [" + purchasableItem.itemName + "]");

                GamebaseManager.requestPurchase(getActivity(), purchasableItem.itemSeq, new GamebaseDataCallback<PurchasableReceipt>() {
                    @Override
                    public void onCallback(PurchasableReceipt data, GamebaseException exception) {

                        if (Gamebase.isSuccess(exception)) {

                            Gamebase.Util.showToast(getActivity(), "Success Purchase : " + data.toString(), Toast.LENGTH_SHORT);
                            AppUtil.dismissProgressDialog(getActivity());

                            Logger.d(TAG, data.paymentSeq);
                            Logger.d(TAG, data.currency);
                            Logger.d(TAG, data.purchaseToken);
                            Logger.d(TAG, String.valueOf(data.itemSeq));
                            Logger.d(TAG, String.valueOf(data.price));
                        } else {

                            Gamebase.Util.showAlert(getActivity(), "Error", exception.toJsonString());
                            AppUtil.dismissProgressDialog(getActivity());

                            Logger.d(TAG, exception.toJsonString());
                            Logger.d(TAG, exception.getDomain());
                            Logger.d(TAG, String.valueOf(exception.getCode()));
                            Logger.d(TAG, exception.getMessage());
                            Logger.d(TAG, exception.getDetailDomain());
                            Logger.d(TAG, String.valueOf(exception.getDetailCode()));
                            Logger.d(TAG, exception.getDetailMessage());

                        }
                    }
                });
            }
        });
    }

    private void itemList() {

        Button buttonItemList = mCurrentView.findViewById(R.id.button_IAPItemList);

        buttonItemList.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

                AppUtil.showProgressDialog(getActivity(), "Get Item List... ");

                GamebaseManager.requestItemList(getActivity(), new GamebaseDataCallback<List<PurchasableItem>>() {
                    @Override
                    public void onCallback(List<PurchasableItem> data, GamebaseException exception) {

                        if (Gamebase.isSuccess(exception)) {

                            Logger.d(TAG, data.toString());

                            List<String> viewItemList = new ArrayList<String>();

                            if (data.size() > 0) {
                                for (int i = 0; i < data.size(); i++) {
                                    PurchasableItem item = data.get(i);

                                    StringBuilder sb = new StringBuilder();
                                    sb.append(item.itemName);
                                    sb.append("(");
                                    sb.append(String.valueOf(item.itemSeq));
                                    sb.append(")");
                                    sb.append(" : ");
                                    sb.append(String.valueOf(item.price));
                                    sb.append(" " + item.currency);
                                    sb.append(" [" + String.valueOf(item.marketItemId) + "]");

                                    viewItemList.add(sb.toString());
                                }
                            } else {
                                viewItemList.add("Empty");
                            }

                            SimpleSelectItemDialog.show(getActivity(), "IAP Item List", viewItemList, null, null, null);
                            AppUtil.dismissProgressDialog(getActivity());
                        } else {
                            Gamebase.Util.showAlert(getActivity(), "requestItemListPurchasable error", exception.toJsonString());
                            AppUtil.dismissProgressDialog(getActivity());

                            Logger.d(TAG, exception.toJsonString());
                        }
                    }
                });
            }
        });
    }

    private void itemListOfNotConsumed() {

        Button buttonItemListOfNotConsumed = mCurrentView.findViewById(R.id.button_notConsumed);

        buttonItemListOfNotConsumed.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

                AppUtil.showProgressDialog(getActivity(), "Get NotConsumed Item List ... ");

                GamebaseManager.requestNotConsumedItems(getActivity(), new GamebaseDataCallback<List<PurchasableReceipt>>() {
                    @Override
                    public void onCallback(List<PurchasableReceipt> data, GamebaseException exception) {

                        if (Gamebase.isSuccess(exception)) {

                            Logger.d(TAG, data.toString());

                            List<String> viewItemList = new ArrayList<String>();
                            if (data.size() > 0) {
                                for (int i = 0; i < data.size(); i++) {
                                    viewItemList.add(purchasableReceiptToString(data.get(i)));
                                }
                            } else {
                                viewItemList.add("Empty");
                            }

                            SimpleSelectItemDialog.show(getActivity(), "Item List Of Not Consumed", viewItemList, null, null, null);
                            AppUtil.dismissProgressDialog(getActivity());
                        } else {
                            Logger.d(TAG, exception.toJsonString());
                            Gamebase.Util.showAlert(getActivity(), "requestItemListOfNotConsumed error", exception.toJsonString());
                            AppUtil.dismissProgressDialog(getActivity());
                        }
                    }
                });
            }
        });
    }

    private void requestRetryTransaction() {

        Button buttonRetryTransaction = mCurrentView.findViewById(R.id.button_retryTransaction);

        buttonRetryTransaction.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

                AppUtil.showProgressDialog(getActivity(), "Get Retry Transaction List ...");

                GamebaseManager.requestRetryTransaction(getActivity(), new GamebaseDataCallback<PurchasableRetryTransactionResult>() {
                    @Override
                    public void onCallback(final PurchasableRetryTransactionResult data, GamebaseException exception) {

                        if (Gamebase.isSuccess(exception)) {

                            Logger.d(TAG, data.toString());

                            final List<PurchasableReceipt> failList = data.failList;
                            final List<PurchasableReceipt> successList = data.successList;

                            List<String> list = new ArrayList<String>();
                            list.add("FailList");
                            list.add("SuccessList");

                            AppUtil.dismissProgressDialog(getActivity());

                            SimpleSelectItemDialog.show(getActivity(), "Select List", list, new DialogInterface.OnClickListener() {
                                @Override
                                public void onClick(DialogInterface dialog, int which) {
                                    if (which == 0) {
                                        List<String> failListString = new ArrayList<String>();

                                        if (failList.size() > 0) {
                                            for (int i = 0; i < failList.size(); i++) {

                                                failListString.add(purchasableReceiptToString(failList.get(i)));

                                            }
                                        } else {
                                            failListString.add("Empty");
                                        }

                                        SimpleSelectItemDialog.show(getActivity(), "Fail List", failListString, null, null, null);
                                    } else {
                                        List<String> successListString = new ArrayList<String>();
                                        if (successList.size() > 0) {
                                            for (int i = 0; i < successList.size(); i++) {

                                                successListString.add(purchasableReceiptToString(successList.get(i)));
                                            }
                                        } else {
                                            successListString.add("Empty");
                                        }
                                        SimpleSelectItemDialog.show(getActivity(), "Success List", successListString, null, null, null);

                                    }
                                }
                            }, null, null);
                        } else {
                            Logger.d(TAG, exception.toJsonString());
                            Gamebase.Util.showAlert(getActivity(), "requestRetryTransaction error", exception.toJsonString());
                        }
                    }
                });
            }
        });
    }

    private String purchasableReceiptToString(PurchasableReceipt item) {

        StringBuilder sb = new StringBuilder();
        sb.append(item.purchaseToken);
        sb.append(" : " + String.valueOf(item.itemSeq));
        sb.append(" : " + String.valueOf(item.price));
        sb.append(" " + item.currency);

        return sb.toString();
    }

}
